@@ -1,10 +1,11 @@
 """
 pymtheg: A Python script to share songs from Spotify/YouTube as a 15 second clip
--------
+
+--------------------------------------------------------------------------------
 
 This is free and unencumbered software released into the public domain.
 
------------------------------------------------------------------------------------------
+-----------------------------------------------------------------------
 
 Anyone is free to copy, modify, publish, use, compile, sell, or distribute this software,
 either in source code form or as a compiled binary, for any purpose, commercial or
@@ -25,12 +26,13 @@ FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 For more information, please refer to <http://unlicense.org/>
 """
 
-from typing import Iterable, List, NamedTuple, Optional, Type, Union
+from typing import Iterable, List, Literal, NamedTuple, Optional, Tuple, Union
 
+from argparse import ArgumentParser, RawTextHelpFormatter
 from tempfile import TemporaryDirectory
-from argparse import ArgumentParser
 from traceback import print_tb
 from datetime import datetime
+from random import randint
 from pathlib import Path
 from shutil import move
 from json import loads
@@ -43,49 +45,57 @@ FFARGS: str = (
     "-hide_banner -loglevel error -c:a aac -c:v libx264 -pix_fmt yuv420p "
     "-tune stillimage -vf scale='iw+mod(iw,2):ih+mod(ih,2):flags=neighbor'"
 )
+OUT: str = "{artists} - {title}"
+TIMESTAMP_FORMAT: str = " ({cs}{cer})"
+CLIP_START: str = "0"
+CLIP_END: str = "+15"
 
 premsg_info = "[dim]pymtheg: [/dim][bold cyan]info[/bold cyan][dim]:[/]"
 premsg_error = "[dim]pymtheg: [/dim][bold red]error[/bold red][dim]:[/]"
 
 
-class EndTimestamp(NamedTuple):
+class Timestamp(NamedTuple):
     """
-    end timestamp named tuple
+    timestamp named tuple
 
+    type: Literal[0] | Literal[1]
+        0 if start timestamp; 1 if end timestamp
     ss: int
-        timestamp of clip end in seconds
+        timestamp in seconds
+    random: bool
+        is timestamp random
     relative: bool = False
-        is ss relative to clip start?
+        is timestamp relative
     """
 
+    type: Union[Literal[0], Literal[1]]
     ss: int
+    random: bool = False
     relative: bool = False
 
-    def __str__(self):
+    def __str__(self) -> str:
         return ("+" if self.relative else "") + str(self.ss)
 
 
 class Behaviour(NamedTuple):
-    """
-    typed command line argument tuple
-    """
+    """typed command line argument tuple"""
 
-    query: str
+    queries: List[str]
     dir: Path
-    out: Optional[Path]
+    out: str
+    timestamp_format: str
+    ext: str
     sdargs: List[str]
     ffargs: List[str]
-    clip_start: int
-    clip_end: EndTimestamp
+    clip_start: Timestamp
+    clip_end: Timestamp
     image: Optional[Path]
     use_defaults: bool
     yes: bool
 
 
 def main() -> None:
-    """
-    pymtheg entry point
-    """
+    """pymtheg entry point"""
     console = Console()
     bev = get_args(console)
 
@@ -98,7 +108,7 @@ def main() -> None:
             spotdl_proc = invocate(
                 console=console,
                 name="spotdl",
-                args=[bev.query] + bev.sdargs,
+                args=bev.queries + ["--path-template", f"{bev.out}.{{ext}}"] + bev.sdargs,
                 cwd=tmpdir,
                 errcode=2,
                 capture_output=True,
@@ -112,25 +122,44 @@ def main() -> None:
             if song_path.suffix not in [".m4a", ".ogg", ".flac", ".mp3", ".wav", ".opus"]:
                 continue
 
+            # duration retrieval
+            with console.status(f"[dim]status: probe song duration[/]", spinner="arc"):
+                proc = invocate(
+                    console=console,
+                    name="ffprobe",
+                    args=[
+                        "-print_format",
+                        "json",
+                        "-show_entries",
+                        "format=duration",
+                        song_path,
+                    ],
+                    capture_output=True,
+                )
+                song_duration: int = int(
+                    loads(proc.stdout)["format"]["duration"].split(".")[0]
+                )
+
             if processed == 0:
                 # print timestamp format/using default message on first song
                 if bev.use_defaults:
                     console.print(
-                        f"{premsg_info} using defaults, clip start will be 0 and clip "
-                        f" end will be {bev.clip_end}\n"
+                        f'{premsg_info} using defaults, clip start is "{bev.clip_start}"'
+                        f' and clip end is "{bev.clip_end}"\n'
                     )
 
                 else:
                     console.print(f"{premsg_info} enter timestamps in format \[hh:mm:]ss")
+                    console.print('               timestamp can be "*" for random')
                     console.print(
-                        "               end timestamp can be relative, prefix with '+'"
+                        '               end timestamp can be relative, prefix with "+"'
                     )
                     console.print(
                         f"               press enter to use given defaults "
-                        f"({bev.clip_start}, {bev.clip_end})\n"
+                        f'("{bev.clip_start}", "{bev.clip_end}")\n'
                     )
 
-            console.print(f"- [bold]{song_path.stem}[/]")
+            console.print(f"- [bold]{song_path.stem}[/] ({to_timestamp(song_duration)})")
 
             # generate query/info messages
             _msg_format = "    {}: "
@@ -165,42 +194,144 @@ def main() -> None:
             info_notice = _msg_format.format(_info_notice.rjust(_longest_msg_len))
             indent = len(_msg_format) - 2 + _longest_msg_len
 
-            # duration retrieval
-            with console.status(
-                f"[dim]{info_status}probe song duration[/]", spinner="arc"
-            ):
-                proc = invocate(
-                    console=console,
-                    name="ffprobe",
-                    args=[
-                        "-print_format",
-                        "json",
-                        "-show_entries",
-                        "format=duration",
-                        song_path,
-                    ],
-                    capture_output=True,
-                )
-                song_duration: int = int(
-                    loads(proc.stdout)["format"]["duration"].split(".")[0]
-                )
-
-            # construct paths
+            # construct working paths
             song_path = song_path.absolute()
             song_clip_path = tmpdir.joinpath(f"{song_path.stem}_clip.mp3").absolute()
             song_cover_path = tmpdir.joinpath(f"{song_path.stem}_cover.png").absolute()
             video_clip_path = tmpdir.joinpath(f"{song_path.stem}_clip.mp4").absolute()
-            out_path: Path = bev.dir.joinpath(f"{song_path.stem}.mp4").absolute()
 
-            if bev.out is not None:
-                out_path = bev.out
+            # get timestamps
+            start_timestamp, end_timestamp = parse_timestamps(
+                bev.clip_start, bev.clip_end, duration=song_duration
+            )
 
-            elif (
+            if bev.clip_end.relative:
+                end_timestamp += start_timestamp
+
+            if end_timestamp == -1:
+                end_timestamp = song_duration
+
+            if not bev.use_defaults:
+                # timestamp prompt
+                while True:
+                    _start_timestamp: Optional[Timestamp] = None
+                    _end_timestamp: Optional[Timestamp] = None
+
+                    # starting timestamp
+                    while True:
+                        cs_response = input(query_clip_start)
+
+                        if cs_response != "":
+                            _start_timestamp = check_timestamp(0, cs_response)
+
+                            if _start_timestamp is None:
+                                # invalid format
+                                console.print(
+                                    "[dim][red]"
+                                    + (" " * indent)
+                                    + ("^" * len(cs_response))
+                                    + "[/dim][bold] invalid timestamp",
+                                )
+
+                            else:
+                                if _start_timestamp.ss > song_duration:
+                                    # invalid, timestamp >= song duration
+                                    console.print(
+                                        "[dim][red]"
+                                        + (" " * indent)
+                                        + ("^" * len(cs_response))
+                                        + "[/dim][bold] timestamp exceeds song duration",
+                                    )
+
+                                else:
+                                    break
+
+                        else:
+                            _start_timestamp = bev.clip_start
+                            break
+
+                    # ending timestamp
+                    while True:
+                        ce_response = input(query_clip_end)
+
+                        if ce_response != "":
+                            _end_timestamp = check_timestamp(1, ce_response)
+
+                            if _end_timestamp is None:
+                                # invalid format
+                                console.print(
+                                    "[dim][red]"
+                                    + (" " * indent)
+                                    + ("^" * len(cs_response))
+                                    + "[/dim][bold] invalid timestamp",
+                                )
+
+                            else:
+                                break
+
+                        else:
+                            _end_timestamp = bev.clip_end
+                            break
+
+                    assert isinstance(_start_timestamp, Timestamp)  # type: ignore
+                    assert isinstance(_end_timestamp, Timestamp)  # type: ignore
+
+                    # parse timestamps
+                    start_timestamp, end_timestamp = parse_timestamps(
+                        _start_timestamp, _end_timestamp, duration=song_duration
+                    )
+
+                    # confirm timestamps
+                    if bev.yes:
+                        break
+
+                    # dont prompt confirmation if defaults were used
+                    if not (cs_response == "" and ce_response == ""):
+                        console.print(
+                            "{premsg}clip duration: {start} -> {end} ({duration}s)".format(
+                                premsg=info_notice,
+                                start=to_timestamp(start_timestamp),
+                                end=to_timestamp(end_timestamp),
+                                duration=end_timestamp - start_timestamp,
+                            )
+                        )
+                        confirmation_response = input(
+                            f"{' ' * indent}confirm? [y/n] (y) "
+                        ).lower()
+
+                        if confirmation_response == "y" or confirmation_response == "":
+                            break
+
+                        else:
+                            pass
+
+                    else:
+                        break
+
+            elif start_timestamp > song_duration:
+                console.print(f"{info_notice}skipping song")
+                processed += 1
+                break
+
+            # construct and confirm output path
+            out_path: Path = bev.dir.joinpath(
+                "{name}{timestamp}.{ext}".format(
+                    name=song_path.stem,
+                    timestamp=tf_format(
+                        string=bev.timestamp_format,
+                        clip_start=start_timestamp,
+                        clip_end=end_timestamp,
+                    ),
+                    ext=bev.ext,
+                )
+            ).absolute()
+
+            if (
                 # no -o specified and out_path exists
                 out_path.exists()
                 and bev.yes is False
             ):
-                console.print(f"{info_notice}'{out_path.name}' exists in output dir.")
+                console.print(f'{info_notice}"{out_path.name}" exists in output dir.')
                 overwrite_response = input(
                     f"{' ' * indent}overwrite? ([y]es/[n]o/[c]hange) "
                 ).lower()
@@ -226,97 +357,6 @@ def main() -> None:
                     processed += 1
                     break
 
-            # get timestamps
-            start_timestamp = bev.clip_start
-            end_timestamp: int = bev.clip_end.ss
-
-            if bev.clip_end.relative:
-                end_timestamp += start_timestamp
-
-            if end_timestamp == -1:
-                end_timestamp = song_duration
-
-            if not bev.use_defaults:
-                while True:
-                    # starting timestamp
-                    while True:
-                        cs_response = input(query_clip_start)
-
-                        if cs_response != "":
-                            _start_timestamp = parse_timestamp(cs_response)
-
-                            if _start_timestamp is None:
-                                # invalid format
-                                console.print(
-                                    "[dim][red]"
-                                    + (" " * indent)
-                                    + ("^" * len(cs_response))
-                                    + "[/dim][bold] invalid timestamp",
-                                )
-
-                            elif _start_timestamp >= song_duration:
-                                # invalid, timestamp >= song duration
-                                console.print(
-                                    "[dim][red]"
-                                    + (" " * indent)
-                                    + ("^" * len(cs_response))
-                                    + "[/dim][bold] timestamp exceeds song duration",
-                                )
-
-                            else:
-                                # valid, continue
-                                end_timestamp = (
-                                    end_timestamp - start_timestamp
-                                ) + _start_timestamp
-                                start_timestamp = _start_timestamp
-                                break
-
-                        elif cs_response == "":
-                            break
-
-                    # ending timestamp
-                    while True:
-                        ce_response = input(query_clip_end)
-                        if ce_response != "":
-                            _end_timestamp = parse_timestamp(
-                                ce_response,
-                                relative_to=start_timestamp,
-                                song_duration=song_duration,
-                            )
-
-                            if _end_timestamp is None:
-                                # reprompt if invalid
-                                console.print(
-                                    (" " * indent) + ("^" * len(ce_response)),
-                                    "invalid timestamp",
-                                )
-
-                            else:
-                                end_timestamp = _end_timestamp
-                                break
-
-                        elif ce_response == "":
-                            break
-
-                    # confirm timestamps
-                    if bev.yes:
-                        break
-
-                    # dont prompt confirmation if defaults were used
-                    if not (cs_response == "" and ce_response == ""):
-                        confirmation_response = input(
-                            f"{info_notice}confirm timestamps? [y/n] (y) "
-                        ).lower()
-
-                        if confirmation_response == "y" or confirmation_response == "":
-                            break
-
-                        else:
-                            pass
-
-                    else:
-                        break
-
             # clip audio
             with console.status(f"[dim]{info_status}clip audio[/]", spinner="arc"):
                 invocate(
@@ -376,7 +416,7 @@ def main() -> None:
                     errcode=3,
                 )
 
-                move(video_clip_path, out_path)
+                move(str(video_clip_path), str(out_path))
 
             processed += 1
 
@@ -397,7 +437,7 @@ def main() -> None:
         exit(1)
 
 
-def part_of_day():
+def part_of_day() -> str:
     """
     used to greet user goodbye
 
@@ -415,48 +455,145 @@ def part_of_day():
     )
 
 
-def parse_timestamp(
-    ts: str,
-    relative_to: Optional[int] = None,
-    song_duration: Optional[int] = None,
-) -> Optional[int]:
+def check_timestamp(type: Union[Literal[0], Literal[1]], ts: str) -> Optional[Timestamp]:
     """
-    parse user-submitted timestamp
+    checks timestamps for timestamp retrieval and command line argument validation
 
     ts: str
-        timestamp following [hh:mm:]ss format (e.g. 2:49, 5:18:18)
-    relative_to: Optional[int] = None
-        used for relative end timestamps
-    song_duration: Optional[int] = None
-        used for the -1 relative end timestamp
+        timestamp string
+    type: Literal[0] | Literal[1]
+        0 if start timestamp; 1 if end timestamp
+
+    returns a Timestamp object if check was successful else None
     """
-    timestamp = 0
+    ts = ts.strip()
 
-    if ts == "-1" and song_duration is not None:
-        return song_duration
+    if ts == "*":
+        return Timestamp(type=type, ss=0, random=True)
 
-    elif ts.startswith("+") and relative_to is not None:
-        ts = ts[1:]
-        timestamp += relative_to
-
-    sts = ts.split(":")  # split time stamp (hh:mm:ss)
-    sts.reverse()  # (ss:mm:hh)
-
-    tu_conv = [1, 60, 3600]  # time unit conversion
-    total_ss = 0  # total seconds
-
-    if len(sts) < 4:
-        for tu, tu_c in zip(sts, tu_conv):
-            if tu.isnumeric():
-                total_ss += int(tu) * tu_c
-
-            else:
-                return None
-
-        return timestamp + total_ss
+    elif ts == "-1":
+        return Timestamp(type=type, ss=-1)
 
     else:
-        return None
+        relative: bool
+        if ts.startswith("+"):
+            if type == 0:  # relative timestamps in start timestamp are not allowed
+                return None
+
+            relative = True
+            ts = ts[1:]
+
+        else:
+            relative = False
+
+        sts = ts.split(":")  # split time stamp (hh:mm:ss)
+        sts.reverse()  # (ss:mm:hh)
+
+        tu_conv = [1, 60, 3600]  # time unit conversion
+        total_ss = 0  # total seconds
+
+        if len(sts) < 4:
+            for tu, tu_c in zip(sts, tu_conv):
+                if tu.isnumeric():
+                    total_ss += int(tu) * tu_c
+
+                else:
+                    return None
+
+            return Timestamp(type=type, ss=total_ss, relative=relative)
+
+        else:
+            return None
+
+
+def parse_timestamps(start: Timestamp, end: Timestamp, duration: int) -> Tuple[int, int]:
+    """
+    parses start timestamp and end timestamp into absolute seconds
+
+    start: Timestamp
+        start timestamp
+    end: Timestamp
+        end timestamp
+    duration: int
+        song duration in seconds
+
+    returns song start and song end respectively, in seconds
+    """
+    ts_start: int
+    ts_end: int
+
+    if start.random and end.random:
+        ts_start = randint(0, duration)
+        ts_end = randint(ts_start, duration)
+
+    elif start.random:
+        ensure_random = end.ss if end.relative else 0
+        ts_start = randint(0, duration - ensure_random)
+        ts_end = ts_start + end.ss
+
+    elif end.random:
+        ts_start = start.ss
+        ts_end = randint(start.ss, duration)
+
+    else:
+        ts_start = start.ss
+        ts_end = (start.ss + end.ss) if end.relative else end.ss
+
+    return (ts_start, ts_end)
+
+
+def to_timestamp(ts: int) -> str:
+    """returns a [(h*):mm:]ss timestamp string from `ts: int`"""
+    _mm = ts // 60
+    hh = _mm // 60
+    mm = _mm - hh * 60
+    ss = ts % 60
+
+    return ":".join(
+        [str(u).rjust(2, "0") for u in (hh, mm) if u != 0] + [str(ss).rjust(2, "0")]
+    ).lstrip("0")
+
+
+def tf_format(string: str, clip_start: int, clip_end: int) -> str:
+    """
+    formats a string with clip information, returns result
+
+    clip_start: int
+        clip start in seconds
+    clip_end: int
+        clip end in seconds
+    """
+
+    def ts_format(ts: int) -> str:
+        """nested function represent `ts: int` as [(h*)mm]ss, returns result"""
+        _mm = ts // 60
+        hh = _mm // 60
+        mm = _mm - hh * 60
+        ss = ts % 60
+
+        result = ""
+
+        for index, unit in enumerate([ss] + [u for u in (mm, hh) if u != 0]):
+            if index < 2:  # ss or mm
+                result = str(unit).rjust(2, "0") + result
+            else:
+                result = str(unit) + result
+
+        return result.lstrip("0")
+
+    replaceables = (
+        ("{cs}", ts_format(clip_start)),
+        ("{css}", clip_start),
+        ("{ce}", ts_format(clip_end)),
+        ("{ces}", clip_end),
+        ("{cer}", f"+{clip_end - clip_start}"),
+    )
+
+    for placeholder, value in replaceables:
+        if placeholder in string:
+            string = string.replace(placeholder, str(value))
+
+    return string
 
 
 def invocate(
@@ -531,28 +668,104 @@ def invocate(
 
 
 def get_args(console: Console) -> Behaviour:
-    """
-    parse and validate arguments
-    """
+    """parse and validate arguments"""
     # parse
     parser = ArgumentParser(
         prog="pymtheg",
         description=(
             "a python script to share songs from Spotify/YouTube as a 15 second clip"
         ),
-        epilog=f'ffargs default: "{FFARGS}"',
+        epilog=f"""querying:
+  queries are passed onto spotdl, and thus must be any one of the following:
+    1. text
+      "<query>"
+      e.g. "thundercat - them changes"
+    2. spotify track/album url
+      "<url>"
+      e.g. "https://open.spotify.com/track/..."
+    3. youtube source + spotify metadata
+      "<youtube url>|<spotify url>"
+      e.g. "https://www.youtube.com/watch?v=...|https://open.spotify.com/track/..."
+
+argument defaults:
+  -f, --ffargs:
+    "{FFARGS}"
+  -o, --out:
+    "{OUT}"
+  -t, --timestamp-format:
+    "{TIMESTAMP_FORMAT}"
+
+formatting:
+  available placeholders:
+    from spotdl:
+      {{artist}}, {{artists}}, {{title}}, {{album}}, {{playlist}}
+    from pymtheg:
+      {{cs}}
+        clip end as per [(h*)mm]ss
+        e.g. 10648 (1h, 06m, 48s)
+      {{css}}
+        clip end in seconds
+        e.g. 4008 (1h, 6m, 48s -> 4008s)
+      {{ce}}
+        clip end as per [(h*)mm]ss, e.g. 10703 (1h, 07m, 03s)
+      {{ces}}
+        clip end in seconds
+        e.g. 4023 (1h, 07m, 03s -> 4023s)
+      {{cer}}
+        clip end relative to clip start, prefixed with +
+        e.g. +15
+    
+      notes:
+        1. pymtheg placeholders can only be used with `-tf, --timestamp-format`
+        2. "[(h*)mm]ss": seconds and minutes will always be represented as 2
+           digits and will be right adjusted with 0s if needed, unless they are
+           the first shown unit where they may have up to two characters. hours
+           can be represented by any number of characters.
+           e.g. "138:02:09", "1:59:08", "2:05", "6"
+
+examples:
+  1. get a song through a spotify link
+    pymtheg "https://open.spotify.com/track/..."
+  2. get a song through a search query
+    pymtheg "thundercat - them changes"
+  3. get multiple songs through multiple queries
+    pymtheg "https://open.spotify.com/track/..." "<query 2>"
+  4. get a random 15s clip of a song
+    pymtheg "<query>" -cs "*" -ce "+15" -ud 
+
+  note: see querying for more information on queries
+""",
+        formatter_class=RawTextHelpFormatter,
     )
 
-    parser.add_argument("query", help="song/link from spotify/youtube")
+    parser.add_argument("queries", help="song queries (see querying)", nargs="+")
     parser.add_argument(
-        "-d", "--dir", type=Path, help="directory to output to", default=Path.cwd()
+        "-d",
+        "--dir",
+        type=Path,
+        help="directory to output to, formattable (see formatting)",
+        default="",
     )
     parser.add_argument(
         "-o",
         "--out",
         type=Path,
-        help="output file path, overrides directory arg",
-        default=None,
+        help=f"output file name format, formattable (see formatting)",
+        default=OUT,
+    )
+    parser.add_argument(
+        "-tf",
+        "--timestamp-format",
+        type=str,
+        help="timestamp format, formattable (see formatting)",
+        default=TIMESTAMP_FORMAT,
+    )
+    parser.add_argument(
+        "-e",
+        "--ext",
+        type=str,
+        help=f'file extension, defaults to "mp4"',
+        default="mp4",
     )
     parser.add_argument("-sda", "--sdargs", help="args to pass to spotdl", default="")
     parser.add_argument(
@@ -566,8 +779,8 @@ def get_args(console: Console) -> Behaviour:
         "--clip-start",
         help="specify clip start (default 0)",
         dest="clip_start",
-        type=int,
-        default=0,
+        type=str,
+        default=CLIP_START,
     )
     parser.add_argument(
         "-ce",
@@ -575,7 +788,7 @@ def get_args(console: Console) -> Behaviour:
         help="specify clip end (default +15)",
         dest="clip_end",
         type=str,
-        default="+15",
+        default=CLIP_END,
     )
     parser.add_argument(
         "-i", "--image", help="specify custom image", type=Path, default=None
@@ -583,7 +796,7 @@ def get_args(console: Console) -> Behaviour:
     parser.add_argument(
         "-ud",
         "--use-defaults",
-        help="use 0 as clip start and --clip-length as clip end",
+        help="use --clip-start as clip start and --clip-length as clip end",
         dest="use_defaults",
         action="store_true",
         default=False,
@@ -598,48 +811,68 @@ def get_args(console: Console) -> Behaviour:
 
     args = parser.parse_args()
 
-    # validate (1)
-    end_timestamp = parse_timestamp(
-        # use dumb values, because who knows if somebody sets it to -1
-        args.clip_end,
-        relative_to=0,
-        song_duration=-1,
-    )
+    # validate clip start/end
+    start_timestamp = check_timestamp(0, args.clip_start)
+    end_timestamp = check_timestamp(1, args.clip_end)
+
+    if start_timestamp is None:
+        console.print(f"{premsg_error} invalid clip start (format: \[hh:mm:]ss)")
+        exit(1)
+
     if end_timestamp is None:
         console.print(
-            f"{premsg_error} invalid clip end (format: [hh:mm:]ss), "
-            "prefix with '+' for relative timestamp"
+            f"{premsg_error} invalid clip end (format: \[hh:mm:]ss), "
+            'prefix with "+" for relative timestamp'
         )
         exit(1)
 
+    # validate formattables to make sure they dont contain illegal placeholders
+    spotdl_replaceables = (
+        "{artist}",
+        "{artists}",
+        "{title}",
+        "{album}",
+        "{playlist}",
+    )
+    for placeholder in spotdl_replaceables:
+        if placeholder in args.timestamp_format:
+            console.print(
+                f"{premsg_error} specified timestamp format string contains illegal "
+                f"placeholder ({placeholder})"
+            )
+            exit(1)
+
+    pymtheg_replaceables = ("{cs}", "{css}", "{ce}", "{ces}", "{cer}")
+    for placeholder in pymtheg_replaceables:
+        if placeholder in str(args.dir):
+            console.print(
+                f"{premsg_error} specified dir format string contains illegal "
+                f"placeholder ({placeholder})"
+            )
+            exit(1)
+
+        if placeholder in str(args.out):
+            console.print(
+                f"{premsg_error} specified out format string contains illegal "
+                f"placeholders ({placeholder})"
+            )
+            exit(1)
+
     bev = Behaviour(
-        query=args.query,
-        dir=args.dir,
+        queries=args.queries,
+        dir=Path(args.dir),
         out=args.out,
+        timestamp_format=args.timestamp_format,
+        ext=args.ext,
         sdargs=args.sdargs.split(),
         ffargs=args.ffargs.split(),
-        clip_start=args.clip_start,
-        clip_end=EndTimestamp(
-            end_timestamp, relative=True if args.clip_end.startswith("+") else False
-        ),
+        clip_start=start_timestamp,
+        clip_end=end_timestamp,
         image=args.image,
         use_defaults=args.use_defaults,
         yes=args.yes,
     )
 
-    # validate (2)
-    if bev.out is not None:
-        if bev.out.is_dir():
-            console.print(f"{premsg_error} output file is a directory")
-            exit(1)
-
-        if bev.out.exists() and bev.yes is False:
-            overwrite_response = console.input(
-                f"{premsg_info} {bev.out} exists, overwrite? [y/n] (n) "
-            )
-            if overwrite_response.lower() != "y":
-                exit(1)
-
     if not bev.dir.exists():
         console.print(f"{premsg_error} output directory is non-existent")
         exit(1)
